#include <fstream>
#include <iostream>
#include <string>

#include <openbabel/mol.h>
#include <openbabel/obconversion.h>
#include <openbabel/forcefield.h>
#include <openbabel/parsmart.h>
#include <openbabel/generic.h>

using namespace std;
using namespace OpenBabel;

int main(int argc, char* argv[]) {
  // 力場の設定
  string ff = "MMFF94";
  OBForceField* pFF = OBForceField::FindForceField(ff);
  if (!pFF) {
    cerr << "指定の力場が見つかりませんでした" << endl;
    return -1;
  }
  pFF->SetLogFile(&cout);
  pFF->SetLogLevel(OBFF_LOGLVL_NONE);

  // ファイルオープン
  ifstream ifs("Donepezil.sdf");
  if (ifs.fail()) {
    cerr << "ファイルが見つかりませんでした" << endl;
    return -1;
  }

  // 参照分子の読み込み
  OBConversion conv;
  conv.SetInAndOutFormats("SDF", "SDF");
  OBMol mol;
  conv.Read(&mol, &ifs);
  ifs.close();

  mol.DeleteHydrogens();  // 水素を外す

  // 母核構造のマッチング
  OBSmartsPattern smp;
  smp.Init("O=C1CCc2c1cccc2");
  smp.Match(mol);
  vector<vector<int> > mlist = smp.GetUMapList();

  // 拘束原子の設定
  OBFFConstraints constraints;
  for (int i = 0; i < mlist[0].size(); i++) {
    constraints.AddAtomConstraint(mlist[0][i]);
  }

  // 分子をセットアップ（部分電荷などの計算）
  if (!pFF->Setup(mol, constraints)) {
    cerr << "分子の設定に失敗しました" << endl;
    return -1;
  }

  // ランダムにコンホメーション発生
  pFF->RandomRotorSearch(10, 0);
  pFF->GetConformers(mol);

  mol.AddHydrogens();  // 水素を付加

  char s[99];
  ofstream ofs("out.sdf");
  for (int i = 0; i < mol.NumConformers(); i++) {
    mol.SetConformer(i);
    if (!pFF->Setup(mol, constraints)) {
      cerr << "分子の設定に失敗しました" << endl;
      return -1;
    }
    pFF->ConjugateGradients(5000);
    pFF->GetCoordinates(mol);
    sprintf(s, "%f", pFF->Energy());
    string Unit = pFF->GetUnit();
    auto pd = new OBPairData;
    pd->SetAttribute("Energy");
    pd->SetValue(s);
    mol.SetData(pd);
    conv.Write(&mol, &ofs);
    mol.DeleteData(pd);
  }
  ofs.close();

  return 0;
}
